use crate::{fold::impl_fold, kw};
use proc_macro2::{Span, TokenStream};
use quote::{quote, quote_spanned, ToTokens};
use syn::{
    parse::{Parse, ParseStream},
    parse_quote_spanned,
    punctuated::Punctuated,
    token::Paren,
    Attribute, Ident, Path, Token, WhereClause, WherePredicate,
};

crate::options! {
    #[options = TypeOptions]
    pub(crate) enum TypeOption {
        OutlineGenerated(outline_generated),
        ConnectInexact(connect_inexact),
        Target(target, Path),
    }
}

pub(crate) struct WrappedInConst<'a> {
    outer: &'a mut TokenStream,
    span: Span,
    inner: TokenStream,
}

impl<'a> WrappedInConst<'a> {
    pub(crate) fn new(outer: &'a mut TokenStream, span: Span) -> Self {
        Self {
            outer,
            span,
            inner: TokenStream::new(),
        }
    }
    pub(crate) fn inner(&mut self) -> &mut TokenStream {
        &mut self.inner
    }
}

impl Drop for WrappedInConst<'_> {
    fn drop(&mut self) {
        let inner = &self.inner;
        quote_spanned! {self.span=>
            const _: () = {
                #inner
            };
        }
        .to_tokens(self.outer);
    }
}

pub(crate) fn get_target(target: &Option<(kw::target, Paren, Path)>, item_ident: &Ident) -> Path {
    match target {
        Some((_, _, target)) => target.clone(),
        None => item_ident.clone().into(),
    }
}

pub(crate) fn common_derives(span: Span) -> Attribute {
    parse_quote_spanned! {span=>
        #[::fayalite::__std::prelude::v1::derive(
            ::fayalite::__std::fmt::Debug,
            ::fayalite::__std::cmp::Eq,
            ::fayalite::__std::cmp::PartialEq,
            ::fayalite::__std::hash::Hash,
            ::fayalite::__std::marker::Copy,
            ::fayalite::__std::clone::Clone,
        )]
    }
}
