// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information

// TODO: enable:
// #![warn(missing_docs)]

//! [Main Documentation][_docs]

extern crate self as fayalite;

#[doc(hidden)]
pub use std as __std;

#[doc(inline)]
/// The `#[hdl_module]` attribute is applied to a Rust function so that that function creates
/// a [`Module`][`::fayalite::module::Module`] when called.
/// In the function body it will implicitly create a
/// variable [`m: ModuleBuilder`][`module::ModuleBuilder`].
///
/// See [Fayalite Modules][crate::_docs::modules]
pub use fayalite_proc_macros::hdl_module;

#[doc(inline)]
pub use fayalite_proc_macros::hdl;

#[cfg(feature = "unstable-doc")]
pub mod _docs;

// FIXME: finish
//pub mod annotations;
pub mod array;
pub mod bundle;
//pub mod cli;
//pub mod clock;
pub mod enum_;
pub mod expr;
//pub mod firrtl;
pub mod int;
pub mod intern;
//pub mod memory;
//pub mod module;
//pub mod reg;
//pub mod reset;
pub mod source_location;
pub mod ty;
pub mod util;
//pub mod valueless;
//pub mod wire;

// FIXME: switch to real module mod
pub mod module {
    use crate::{
        enum_::{EnumMatchVariantsIter, EnumType},
        expr::{ops::VariantAccess, Expr},
        intern::Interned,
        source_location::SourceLocation,
    };
    use std::marker::PhantomData;

    pub struct NormalModule;

    pub struct ModuleBuilder<ModuleKind>(PhantomData<ModuleKind>);

    impl ModuleBuilder<NormalModule> {
        pub fn enum_match_variants_helper<Enum: EnumType>(
            &mut self,
            _base: Expr<Enum>,
            _source_location: SourceLocation,
        ) -> EnumMatchVariantsIter<Enum> {
            todo!()
        }
    }

    pub struct Scope(());

    pub(crate) struct EnumMatchVariantAndInactiveScopeImpl<T: EnumType> {
        variant_access: Interned<VariantAccess>,
        _phantom: PhantomData<T>,
    }

    impl<T: EnumType> EnumMatchVariantAndInactiveScopeImpl<T> {
        pub(crate) fn activate(self) -> (Interned<VariantAccess>, Scope) {
            (self.variant_access, Scope(()))
        }
        pub(crate) fn variant_access(&self) -> Interned<VariantAccess> {
            self.variant_access
        }
    }

    #[derive(Clone)]
    pub(crate) struct EnumMatchVariantsIterImpl<T>(PhantomData<T>);

    impl<T: EnumType> EnumMatchVariantsIterImpl<T> {
        pub(crate) fn for_variant_index(
            &self,
            _variant_index: usize,
        ) -> EnumMatchVariantAndInactiveScopeImpl<T> {
            todo!()
        }
    }
}
